@@ -1,21 +1,22 @@
 package Mojolicious::Plugin::CGI;
 use Mojo::Base 'Mojolicious::Plugin';
 
-use Mojo::Util 'b64_decode';
 use File::Basename;
 use File::Spec;
-use Sys::Hostname;
 use IO::Pipely 'pipely';
+use Mojo::Util qw(b64_decode encode);
 use POSIX 'WNOHANG';
 use Perl::OSType 'is_os_type';
 use Socket qw(AF_INET inet_aton);
+use Sys::Hostname;
+
 use constant CHECK_CHILD_INTERVAL => $ENV{CHECK_CHILD_INTERVAL} || 0.01;
 use constant DEBUG                => $ENV{MOJO_PLUGIN_CGI_DEBUG};
 use constant IS_WINDOWS           => is_os_type('Windows');
 use constant READ                 => 0;
 use constant WRITE                => 1;
 
-our $VERSION = '0.35';
+our $VERSION = '0.37';
 our %ORIGINAL_ENV = %ENV;
 
 has env => sub { +{%ORIGINAL_ENV} };
@@ -57,7 +58,6 @@ sub _child {
 
   Mojo::IOLoop->reset;
   warn "[CGI:$args->{name}:$$] <<< (@{[$stdin->slurp]})\n" if DEBUG;
-  %ENV = _emulate_environment($c, $args);
   open STDIN, '<', $stdin->path or die "STDIN @{[$stdin->path]}: $!" if -s $stdin->path;
   open STDERR, $STDERR[0], $STDERR[1] or die "STDERR: @$stderr: $!";
   open STDOUT, '>&', fileno $stdout->[WRITE] or die "STDOUT: $!";
@@ -66,6 +66,7 @@ sub _child {
   select STDOUT;
   $| = 1;
 
+  %ENV = _emulate_environment($c, $args);
   $args->{run} ? $args->{run}->($c) : exec $args->{script}
     || die "Could not execute $args->{script}: $!";
 
@@ -112,7 +113,7 @@ sub _emulate_environment {
     GATEWAY_INTERFACE => 'CGI/1.1',
     HTTPS             => $req->is_secure ? 'YES' : 'NO',
     %env_headers,
-    PATH_INFO => '/' . ($c->stash('path_info') || ''),
+    PATH_INFO => '/' . encode('UTF-8', $c->stash('path_info') // ''),
     QUERY_STRING => $c->stash('cgi.query_string') || $req->url->query->to_string,
     REMOTE_ADDR => $tx->remote_address,
     REMOTE_HOST => gethostbyaddr(inet_aton($tx->remote_address || '127.0.0.1'), AF_INET) || '',
@@ -263,7 +264,7 @@ Mojolicious::Plugin::CGI - Run CGI script from Mojolicious
 
 =head1 VERSION
 
-0.35
+0.37
 
 =head1 DESCRIPTION
 
